@@ -1,13 +1,19 @@
-# -*- coding: UTF8 -*-
+# -*- coding: utf-8 -*-
 # Copyright (C) 2001,2002 Python Software Foundation
 # csv package unit tests
 
+from codecs import EncodedFile
 import os
+import sys
 import unittest2 as unittest
 from StringIO import StringIO
 import tempfile
 import unicodecsv as csv
 
+# pypy and cpython differ under which exception is raised under some circumstances
+# e.g. whether a module is written in C or not.
+py_compat_exc = (TypeError, AttributeError)
+
 class Test_Csv(unittest.TestCase):
     """
     Test the underlying C csv parser in ways that are not appropriate
@@ -15,21 +21,21 @@ class Test_Csv(unittest.TestCase):
     in TestDialectRegistry.
     """
     def _test_arg_valid(self, ctor, arg):
-        self.assertRaises(TypeError, ctor)
-        self.assertRaises(TypeError, ctor, None)
-        self.assertRaises(TypeError, ctor, arg, bad_attr = 0)
-        self.assertRaises(TypeError, ctor, arg, delimiter = 0)
-        self.assertRaises(TypeError, ctor, arg, delimiter = 'XX')
+        self.assertRaises(py_compat_exc, ctor)
+        self.assertRaises(py_compat_exc, ctor, None)
+        self.assertRaises(py_compat_exc, ctor, arg, bad_attr = 0)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter = 0)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter = 'XX')
         self.assertRaises(csv.Error, ctor, arg, 'foo')
-        self.assertRaises(TypeError, ctor, arg, delimiter=None)
-        self.assertRaises(TypeError, ctor, arg, delimiter=1)
-        self.assertRaises(TypeError, ctor, arg, quotechar=1)
-        self.assertRaises(TypeError, ctor, arg, lineterminator=None)
-        self.assertRaises(TypeError, ctor, arg, lineterminator=1)
-        self.assertRaises(TypeError, ctor, arg, quoting=None)
-        self.assertRaises(TypeError, ctor, arg,
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter=None)
+        self.assertRaises(py_compat_exc, ctor, arg, delimiter=1)
+        self.assertRaises(py_compat_exc, ctor, arg, quotechar=1)
+        self.assertRaises(py_compat_exc, ctor, arg, lineterminator=None)
+        self.assertRaises(py_compat_exc, ctor, arg, lineterminator=1)
+        self.assertRaises(py_compat_exc, ctor, arg, quoting=None)
+        self.assertRaises(py_compat_exc, ctor, arg,
                           quoting=csv.QUOTE_ALL, quotechar='')
-        self.assertRaises(TypeError, ctor, arg,
+        self.assertRaises(py_compat_exc, ctor, arg,
                           quoting=csv.QUOTE_ALL, quotechar=None)
 
     def test_reader_arg_valid(self):
@@ -50,10 +56,10 @@ class Test_Csv(unittest.TestCase):
         self.assertEqual(obj.dialect.skipinitialspace, False)
         self.assertEqual(obj.dialect.strict, False)
         # Try deleting or changing attributes (they are read-only)
-        self.assertRaises(TypeError, delattr, obj.dialect, 'delimiter')
-        self.assertRaises(TypeError, setattr, obj.dialect, 'delimiter', ':')
-        self.assertRaises(AttributeError, delattr, obj.dialect, 'quoting')
-        self.assertRaises(AttributeError, setattr, obj.dialect,
+        self.assertRaises(py_compat_exc, delattr, obj.dialect, 'delimiter')
+        self.assertRaises(py_compat_exc, setattr, obj.dialect, 'delimiter', ':')
+        self.assertRaises(py_compat_exc, delattr, obj.dialect, 'quoting')
+        self.assertRaises(py_compat_exc, setattr, obj.dialect,
                           'quoting', None)
 
     def test_reader_attrs(self):
@@ -766,7 +772,10 @@ class TestArrayWrites(unittest.TestCase):
         try:
             writer = csv.writer(fileobj, dialect="excel")
             writer.writerow(a)
-            expected = ",".join([str(i) for i in a])+"\r\n"
+            float_repr = str
+            if sys.version_info >= (2, 7, 3):
+                float_repr = repr
+            expected = ",".join([float_repr(i) for i in a])+"\r\n"
             fileobj.seek(0)
             self.assertEqual(fileobj.read(), expected)
         finally:
@@ -782,7 +791,10 @@ class TestArrayWrites(unittest.TestCase):
         try:
             writer = csv.writer(fileobj, dialect="excel")
             writer.writerow(a)
-            expected = ",".join([str(i) for i in a])+"\r\n"
+            float_repr = str
+            if sys.version_info >= (2, 7, 3):
+                float_repr = repr
+            expected = ",".join([float_repr(i) for i in a])+"\r\n"
             fileobj.seek(0)
             self.assertEqual(fileobj.read(), expected)
         finally:
@@ -807,14 +819,40 @@ class TestArrayWrites(unittest.TestCase):
 
 class TestUnicode(unittest.TestCase):
     def test_unicode_read(self):
-        import codecs
-        f = codecs.EncodedFile(StringIO("Martin von Löwis,"
-                                        "Marc André Lemburg,"
-                                        "Guido van Rossum,"
-                                        "François Pinard\r\n"),
-                               data_encoding='iso-8859-1')
+        f = EncodedFile(StringIO("Martin von Löwis,"
+                                 "Marc André Lemburg,"
+                                 "Guido van Rossum,"
+                                 "François Pinard\r\n"),
+                                 data_encoding='iso-8859-1')
         reader = csv.reader(f)
         self.assertEqual(list(reader), [[u"Martin von Löwis",
                                          u"Marc André Lemburg",
                                          u"Guido van Rossum",
-                                         u"François Pinard"]])
\ No newline at end of file
+                                         u"François Pinard"]])
+
+
+class TestUnicodeErrors(unittest.TestCase):
+    def test_encode_error(self):
+        fd = StringIO()
+        writer = csv.writer(fd, encoding='cp1252', errors='xmlcharrefreplace')
+        writer.writerow(['hello', unichr(2603)])
+        self.assertEqual(fd.getvalue(), 'hello,&#2603;\r\n')
+
+    def test_encode_error_dictwriter(self):
+        fd = StringIO()
+        dw = csv.DictWriter(fd, ['col1'], encoding='cp1252', errors='xmlcharrefreplace')
+        dw.writerow({'col1': unichr(2604)})
+        self.assertEqual(fd.getvalue(), '&#2604;\r\n')
+
+    def test_decode_error(self):
+        """Make sure the specified error-handling mode is obeyed on readers."""
+        file = EncodedFile(StringIO('Löwis,2,3'), data_encoding='iso-8859-1')
+        reader = csv.reader(file, encoding='ascii', errors='ignore')
+        self.assertEqual(list(reader)[0][0], 'Lwis')
+
+    def test_decode_error_dictreader(self):
+        """Make sure the error-handling mode is obeyed on DictReaders."""
+        file = EncodedFile(StringIO('name,height,weight\nLöwis,2,3'),
+                           data_encoding='iso-8859-1')
+        reader = csv.DictReader(file, encoding='ascii', errors='ignore')
+        self.assertEqual(list(reader)[0]['name'], 'Lwis')
